from __future__ import annotations

from pathlib import Path

import pymediainfo
from pyd2v import D2V

from nfog.tracks.BaseTrack import BaseTrack


class Video(BaseTrack):
    DYNAMIC_RANGE_MAP = {
        "SMPTE ST 2086": "HDR10",
        "HDR10": "HDR10",
        "SMPTE ST 2094 App 4": "HDR10+",
        "HDR10+": "HDR10+",
        "Dolby Vision": "DV"
    }

    def __init__(self, track: pymediainfo.Track, path: Path):
        super().__init__(track, path)
        # quick shorthands
        self.profile = self._x.format_profile
        self.dar = self._x.other_display_aspect_ratio[0]
        if self._x.framerate_num:
            self.fps = f"{self._x.framerate_num}/{self._x.framerate_den}"
        else:
            self.fps = self._x.frame_rate

    @property
    def codec(self) -> str:
        """
        Get video codec in common P2P simplified form.
        E.g., 'MPEG-2' instead of 'MPEG Video, Version 2'.
        """
        return {
            "MPEG Video": f"MPEG-{(self._x.format_version or '').replace('Version ', '')}"
        }.get(self._x.format, self._x.format)

    @property
    def range(self) -> str:
        """
        Get video range as typical shortname.
        Returns multiple ranges in space-separated format if a fallback range is
        available. E.g., 'DV HDR10'.
        """
        if self._x.hdr_format:
            return " ".join([
                self.DYNAMIC_RANGE_MAP.get(x)
                for x in self._x.hdr_format.split(" / ")
            ])
        elif "HLG" in ((self._x.transfer_characteristics or ""), (self._x.transfer_characteristics_original or "")):
            return "HLG"
        return "SDR"

    @property
    def scan(self) -> str:
        """
        Get video scan type in string form.
        Will accurately check using DGIndex if codec is MPEG-1/2.

        Examples:
            'Interlaced'
            'Progressive'
            If there's information on scan percentages:
            '99.78% Progressive (VST)'
            '0.01% Interlaced (VST)'
        """
        scan_type = self._x.scan_type
        if not scan_type:
            # some videos may not state scan, assume progressive
            scan_type = "Progressive"

        if self.codec in ["MPEG-1", "MPEG-2"]:
            d2v = D2V.load(self._path)
            for ext in ("log", "d2v", "mpg", "mpeg"):
                d2v.path.with_suffix(f".{ext}").unlink(missing_ok=True)

            flags = [
                dict(**flag, vob=d["vob"], cell=d["cell"])
                for d in d2v.data
                for flag in d["flags"]
            ]
            progressive_frames = sum(f["progressive_frame"] for f in flags)
            progressive_percent = (progressive_frames / len(flags)) * 100
            is_constant = progressive_percent in (0.0, 100.0)

            scan_type = ["Interlaced", "Progressive"][progressive_percent >= 50.0]
            if not is_constant:
                scan_type = f"{progressive_percent:.2f}% {scan_type} (VST)"

        return scan_type


__ALL__ = (Video,)



from __future__ import annotations

import re
from collections import defaultdict
from pathlib import Path
from typing import Any, Optional

import pymediainfo
from langcodes import Language


class BaseTrack:
    ALPHA_NUMERIC_RE = re.compile(r"[\W]+")

    """Track to aide in overriding properties of a PyMediaInfo Track instance."""
    def __init__(self, track: pymediainfo.Track, path: Path):
        self._x = track
        self._path = path
        # common shorthands
        self.bitrate = self._x.other_bit_rate[0]

    def __getattr__(self, name: str) -> Any:
        return getattr(self._x, name)

    @property
    def all_properties(self) -> defaultdict[str, Any]:
        """Get all non-callable attributes from this and it's sub-track object."""
        props = defaultdict(lambda: None)

        for obj in (self, self._x):
            for k, v in obj.__dict__.items():
                if k in ("_x", "_path"):
                    continue
                props[k] = v

        for subclass in (BaseTrack, self.__class__):
            for k, v in vars(subclass).items():
                if not isinstance(v, property):
                    continue
                if k in ("all_properties",):
                    continue
                props[k] = getattr(self, k)

        return props

    @property
    def language(self) -> Optional[Language]:
        """
        Return track language as a Language object.
        Returns None if no language is specified, or if set to `und`.
        """
        if self._x.language and self._x.language != "und":
            return Language.get(self._x.language)
        return None

    @property
    def title(self) -> Optional[str]:
        """Get track title in it's simplest form."""
        return (self._x.title or "").strip() or None


__ALL__ = (BaseTrack,)



from __future__ import annotations

import gzip
import logging
from datetime import datetime
from pathlib import Path
from typing import Any, Optional

import click
import jsonpickle
import toml
from click_default_group import DefaultGroup
from pymediainfo import MediaInfo

from nfog import __version__
from nfog.artwork import Artwork
from nfog.config import Directories, Files, config
from nfog.constants import GROUP_SETTINGS
from nfog.templates import Template
from nfog.templates.Group import TemplateGroup


@click.group(
    cls=DefaultGroup,
    default="generate",
    default_if_no_args=True,
    context_settings=GROUP_SETTINGS
)
def cli() -> None:
    """
    \b
    nfog
    Scriptable Database-Driven NFO Generator for Movies and TV.
    https://github.com/rlaphoenix/nfog
    """


@cli.command()
def version() -> None:
    """Shows the version of the project."""
    print(
        f"nfog {__version__}\n"
        f"Scriptable Database-Driven NFO Generator for Movies and TV.\n"
        "https://github.com/rlaphoenix/nfog\n"
        "\n"
        f"Configuration File: {Files.config}\n"
        f"Templates Folder: {Directories.templates}\n"
        f"Artwork Folder: {Directories.artwork}"
    )


@cli.group(
    cls=TemplateGroup,
    context_settings=dict(
        **GROUP_SETTINGS,
        ignore_unknown_options=True,
        default_map=config.get("cli").get("generate", {})
    ))
@click.argument("file", type=Path)
@click.option("-imdb", type=str, default=None, help="IMDb ID (including 'tt').")
@click.option("-tmdb", type=str, default=None, help="TMDB ID (including 'tv/' or 'movie/').")
@click.option("-tvdb", type=int, default=None, help="TVDB ID ('73244' not 'the-office-us').")
@click.option("-a", "--artwork", type=str, default=None, help="Artwork to use.")
@click.option("-s", "--source", type=str, default=None, help="Source information.")
@click.option("-n", "--note", type=str, default=None, help="Notes/special information.")
@click.option("-p", "--preview", type=str, default=None, help="Preview information, typically an URL.")
@click.option("-e", "--encoding", type=str, default="utf8", help="Text-encoding for output, input is always UTF-8.")
@click.pass_context
def generate(
    ctx: click.Context, file: Path, imdb: Optional[str], tmdb: Optional[str], tvdb: Optional[int], **__
) -> None:
    """
    Generates an NFO for the provided file.

    \b
    The file provided should best represent the majority of the release.
    E.g., If Episode 1 and 2 has a fault not found on Episodes 3 onwards, then provide Episode 3.
    """
    if not file.exists():
        raise click.ClickException(f"The provided path ({file}) does not exist.")
    if not file.is_file():
        raise click.ClickException(f"The provided path ({file}) is not to a file.")

    media_info = MediaInfo.parse(file)

    if not imdb:
        imdb = media_info.general_tracks[0].to_data().get("imdb")
        ctx.params["imdb"] = imdb

    if not tmdb:
        tmdb = media_info.general_tracks[0].to_data().get("tmdb")
        ctx.params["tmdb"] = tmdb

    if not tvdb:
        tvdb = media_info.general_tracks[0].to_data().get("tvdb")
        ctx.params["tvdb"] = tvdb

    if not imdb and not tmdb:
        raise click.ClickException(
            "At least an IMDb or TMDB ID must be provided. "
            "TVDB's API is not free so it cannot be used by itself."
        )


@generate.result_callback()
def generator(
    template: Template,
    file: Path,
    artwork: Optional[str] = None,
    encoding: str = "utf8",
    **__
) -> None:
    if artwork:
        fn = Files.artwork(artwork)
        if not fn.exists():
            raise click.ClickException(f"Artwork ({fn}) does not exist.")
        scope: dict[str, Any] = {}
        with fn.open(encoding="utf8") as f:
            code = compile(f.read(), fn, "exec")
            eval(code, scope, scope)
        artwork: Artwork = scope[artwork]
        nfo = artwork.with_template(template)
    else:
        nfo = template.nfo

    out_path: Path = file.parent / f"{template.release_name}{template.file_ext}"
    out_path.write_text(nfo, encoding=encoding)
    print(f"Generated NFO for {template.release_name}")
    print(f" + Saved to: {out_path}")


@cli.command(name="config")
@click.argument("key", type=str, required=False)
@click.argument("value", type=str, required=False)
@click.option("--unset", is_flag=True, default=False, help="Unset/remove the configuration value.")
def config_(key: Optional[str], value: Optional[str], unset: bool) -> None:
    """Manage or view user configuration data."""
    if not key and not value:
        print(toml.dumps(config).rstrip())
        return

    log = logging.getLogger("config")

    tree = key.split(".")
    temp = config
    for t in tree[:-1]:
        if temp.get(t) is None:
            temp[t] = {}
        temp = temp[t]

    if unset:
        if tree[-1] in temp:
            del temp[tree[-1]]
        log.info(f"Unset {key}")
    else:
        if value is None:
            if tree[-1] not in temp:
                raise click.ClickException(f"Key {key} does not exist in the config.")
            print(f"{key}: {temp[tree[-1]]}")
        else:
            temp[tree[-1]] = value
            log.info(f"Set {key} to {repr(value)}")
            Files.config.parent.mkdir(parents=True, exist_ok=True)
            toml.dump(config, Files.config)


@cli.command()
@click.argument("out_dir", type=Path)
def export(out_dir: Path) -> None:
    """Export all configuration, artwork, and templates."""
    if not out_dir or not out_dir.is_dir():
        raise click.ClickException("Save Path must be directory.")
    art = {x.stem: x.read_text(encoding="utf8") for x in Directories.artwork.glob("*.py")}
    tmpl = {x.stem: x.read_text(encoding="utf8") for x in Directories.templates.glob("*.py")}
    json = jsonpickle.dumps({
        "version": 1,
        "config": config,
        "artwork": art,
        "templates": tmpl
    })
    out_dir.mkdir(parents=True, exist_ok=True)
    out_path = out_dir / f"pynfogen.export.{datetime.now().strftime('%Y%m%d-%H%M%S')}.json.gz"
    out_path.write_bytes(gzip.compress(json.encode("utf8")))
    print(f"Successfully exported to: {out_path}")


@cli.command(name="import")
@click.argument("file", type=Path)
def import_(file: Path):
    """
    Import all configuration, artwork, and templates from export.
    The configuration will be overwritten in it's entirety.
    Current artwork and template files will only be overwritten if
    they have the same name.
    """
    if not file or not file.exists():
        raise click.ClickException("File path does not exist.")
    decompress = gzip.open(file).read().decode("utf8")
    json = jsonpickle.decode(decompress)
    Files.config.parent.mkdir(parents=True, exist_ok=True)
    Directories.artwork.mkdir(parents=True, exist_ok=True)
    Directories.templates.mkdir(parents=True, exist_ok=True)
    Files.config.write_text(toml.dumps(json["config"]))
    print("Imported Configuration")
    for name, data in json["artwork"].items():
        path = (Directories.artwork / name).with_suffix(".py")
        path.write_text(data, encoding="utf8")
        print(f"Imported Artwork: {name}")
    for name, data in json["templates"].items():
        path = (Directories.templates / name).with_suffix(".py")
        path.write_text(data, encoding="utf8")
        print(f"Imported Template: {name}")
    print(f"Successfully Imported from {file}!")



from __future__ import annotations

import re
import textwrap
from abc import abstractmethod
from pathlib import Path
from typing import Any, Optional
from urllib.parse import urlparse

import tmdbsimple
from imdb import IMDb
from langcodes import closest_supported_match
from pymediainfo import MediaInfo, Track
from requests import Session

from nfog.config import config
from nfog.tracks import Audio, Subtitle, Video


class Template:
    IMDB_ID_T = re.compile(r"^tt\d{7,8}$")
    TMDB_ID_T = re.compile(r"^(tv|movie)/\d+$")
    TVDB_ID_T = re.compile(r"^\d+$")

    def __init__(
        self,
        file: Path,
        imdb: Optional[str],
        tmdb: Optional[str] = None,
        tvdb: Optional[int] = None,
        source: Optional[str] = None,
        note: Optional[str] = None,
        preview: Optional[str] = None,
        **kwargs: Any
    ):
        self._nfo = []

        self.file = file

        if imdb:
            if not self.IMDB_ID_T.match(imdb):
                raise ValueError(
                    f"The provided IMDb ID ({imdb}) is not valid. Expected e.g., 'tt0487831', 'tt10810424'."
                )
            self.imdb = IMDb().get_movie(imdb.lstrip("tt"), ("main", "episodes"))
        else:
            self.imdb = None

        if tmdb:
            if not self.TMDB_ID_T.match(tmdb):
                raise ValueError(
                    f"The provided TMDB ID ({tmdb}) is not valid. Expected e.g., 'tv/2490', 'movie/14836'."
                )
            if not tmdbsimple.API_KEY:
                raise EnvironmentError("No themoviedb.org api key in config, cannot proceed.")
            self.tmdb = {
                "movie": tmdbsimple.Movies,
                "tv": tmdbsimple.TV
            }[tmdb.split("/")[0]](tmdb.split("/")[1])
        else:
            self.tmdb = None

        if tvdb:
            if not self.TVDB_ID_T.match(str(tvdb)):
                raise ValueError(
                    f"The provided TVDB ID ({tvdb}) is not valid. Expected e.g., '79216', '1395'."
                )
            # tvdb api isn't free, harder to implement
            self.tvdb = tvdb
        else:
            self.tvdb = None

        self.source = source
        self.note = note
        self.preview = preview
        self.args = kwargs

        self.media_info = MediaInfo.parse(self.file)
        self.video_tracks = [Video(x, self.file) for x in self.media_info.video_tracks]
        self.audio_tracks = [Audio(x, self.file) for x in self.media_info.audio_tracks]
        self.text_tracks = [Subtitle(x, self.file) for x in self.media_info.text_tracks]

        self.chapters: Optional[Track] = next(iter(self.media_info.menu_tracks), None)
        if self.chapters:
            self.chapters = {
                ".".join([k.replace("_", ".")[:-3], k[-3:]]): v.strip(":")
                for k, v in self.chapters.to_data().items()
                if f"1{k.replace('_', '')}".isdigit()
            }

        self.primary_lang = next(
            (
                lang.language
                for lang in sorted(self.audio_tracks, key=lambda x: x.streamorder)
                if lang.language
            ),
            # default to first language on IMDb
            self.imdb["language codes"][0] if self.imdb else None
        )

        self._session = None

    @property
    @abstractmethod
    def nfo(self) -> str:
        """Generate the NFO string using Template information."""

    @property
    @abstractmethod
    def release_name(self) -> str:
        """The release name used for the output NFO filename."""

    @property
    @abstractmethod
    def file_ext(self) -> str:
        """The file extension to use when saving this template."""

    @property
    def session(self) -> Session:
        """Get a Request Session."""
        if self._session is not None:
            return self._session

        self._session = Session()
        self._session.headers.update({
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:81.0) Gecko/20100101 Firefox/81.0",
            "Accept-Language": "en-US,en;q=0.5"
        })

        return self._session

    def get_preview_images(self, url: str) -> list[tuple[str, str]]:
        """Get a list of image thumbnail SRCs and full hyperlinks from Gallery url."""
        if not url:
            raise ValueError("Provided URL cannot be empty.")

        domain = ".".join(urlparse(url).netloc.split(".")[-2:])
        supported_domains = ["imgbox.com", "beyondhd.co"]
        if domain not in supported_domains:
            return []

        images = []
        page = self.session.get(url).text
        if domain == "imgbox.com":
            for m in re.finditer(r'src="(https://thumbs2.imgbox.com.+/)(\w+)_b.([^"]+)', page):
                images.append((
                    f"https://imgbox.com/{m.group(2)}",
                    f"{m.group(1)}{m.group(2)}_t.{m.group(3)}"
                ))
        if domain == "beyondhd.co":
            for m in re.finditer(r'/image/([^"]+)"\D+src="(https://.*beyondhd.co/images.+/(\w+).md.[^"]+)', page):
                images.append((
                    f"https://beyondhd.co/image/{m.group(1)}",
                    m.group(2)
                ))

        return images

    def get_banner_image(self, tvdb_id: int, language: str) -> Optional[str]:
        """Get a wide banner image from fanart.tv."""
        if not tvdb_id:
            return None

        api_key = config.get("api-keys").get("fanart-tv")
        if not api_key:
            print("Warning: No fanart.tv api key in config, skipping banner image.")
            return None

        r = self.session.get(f"http://webservice.fanart.tv/v3/tv/{tvdb_id}?api_key={api_key}")
        if r.status_code == 404:
            return None
        res = r.json()

        error = res.get("error message")
        if error:
            if error == "Not found":
                return None
            raise ValueError(f"An unexpected error occurred while calling Fanart.tv, {res}")

        url = next((
            x["url"]
            for x in res.get("tvbanner") or []
            if closest_supported_match(x["lang"], [language], 5)
        ), None)

        return url

    def get_video_summary(self, video: Video) -> str:
        """Get a video track's information in a two-line summary."""
        line_1 = "- "
        if video.language:
            line_1 += f"{video.language.display_name(self.primary_lang)}, "

        line_1 += f"{video.codec} ({video.profile}) "
        line_1 += f"{video.width}x{video.height} ({video.dar}) "
        line_1 += f"@ {video.bitrate}"
        if video.bit_rate_mode:
            line_1 += f" ({video.bit_rate_mode})"

        line_2 = f"  {video.fps} FPS "
        line_2 += f"({video.frame_rate_mode}), "
        line_2 += f"{video.color_space} {video.chroma_subsampling} {video.bit_depth}bps, "
        line_2 += f"{video.range}, {video.scan}"

        return "\n".join([line_1, line_2])

    def get_audio_summary(self, audio: Audio) -> str:
        """Get an audio track's information in a one-line summary."""
        line = "- "
        if audio.language:
            line += f"{audio.language.display_name(self.primary_lang)}, "

        if audio.title:
            line += f"{audio.title}, "
        line += f"{audio.codec} {audio.channels} "
        line += f"@ {audio.bitrate}"
        if audio.bit_rate_mode:
            line += f" ({audio.bit_rate_mode})"

        return line

    def get_subtitle_summary(self, subtitle: Subtitle) -> str:
        """Get a subtitle track's information in a one-line summary."""
        line = "- "
        if subtitle.language:
            line += f"{subtitle.language.display_name(self.primary_lang)}, "

        if subtitle.title:
            line += f"{subtitle.title}, "
        line += subtitle.codec

        return line

    @staticmethod
    def get_chapter_list(chapters: Optional[dict[str, str]]) -> list[str]:
        """Get a list of chapters showing timecode and chapter name."""
        if not chapters:
            return []
        return [
            f"- {k}: {v}"
            for k, v in chapters.items()
        ]

    @staticmethod
    def indented_wrap(text: str, width: int, indent: Optional[str] = None, **kwargs) -> str:
        """
        Wrap text at a specific width and indent.
        The `indent` value will set an initial and subsequent indent. If
        set, it will override any indent options you may have also provided via kwargs.
        """
        if not text:
            return text
        if not width or width < 1:
            raise ValueError(f"An invalid width value of [{width}] was provided.")
        if indent:
            kwargs["initial_indent"] = indent
            kwargs["subsequent_indent"] = indent
        return "\n".join(textwrap.wrap(text, width, **kwargs))

    @staticmethod
    def centered_wrap(text: str, width: int, wrap_width: Optional[int] = None) -> str:
        """
        Center text to a specific width, while also wrapping at another width.
        If wrap_width is not provided, width will be used instead.
        """
        if not text:
            return text
        if not width or width < 1:
            raise ValueError(f"An invalid width value of [{width}] was provided.")
        if wrap_width and wrap_width < width:
            raise ValueError(f"Wrap width [{wrap_width}] cannot be less than centering width [{width}].")
        return "\n".join([
            x.center(width)
            for x in textwrap.wrap(text, wrap_width or width)
        ])

    @staticmethod
    def layout(items: list[str], width: int, spacing: int = 0) -> str:
        """
        Lay out data in a grid with specific widths and spacing.

        Example:
            >>> Template.layout(['1', '2', '3'], width=2)
            12
            3
            >>> Template.layout(['1', '2', '3', '4', '5', '6', '7'], width=4)
            1234
            567
            >>> Template.layout(['1', '2', '3', '4'], width=2, spacing=1)
            1 2

            3 4
        """
        if not items:
            return ""

        grid = [
            items[i:i + width]
            for i in range(0, len(items), width)
        ]

        grid_x_spaced = [(" " * spacing).join(x) for x in grid]
        grid_y_spaced = ("\n" * (spacing + 1)).join(grid_x_spaced)

        return grid_y_spaced


__ALL__ = (Template,)



from __future__ import annotations

from pathlib import Path

import pymediainfo

from nfog.tracks.BaseTrack import BaseTrack


class Subtitle(BaseTrack):
    def __init__(self, track: pymediainfo.Track, path: Path):
        super().__init__(track, path)

    @property
    def codec(self) -> str:
        """
        Get track codec in common P2P simplified form.
        E.g., 'SubRip (SRT)' instead of 'UTF-8'.
        """
        return {
            "UTF-8": "SubRip (SRT)",
        }.get(self._x.format, self._x.format)


__ALL__ = (Subtitle,)



from typing import Any, Optional

import click

from nfog.config import Directories


class TemplateGroup(click.MultiCommand):
    """Lazy-loaded command group of nfo templates."""
    TEMPLATES_DIR = Directories.templates

    def list_commands(self, ctx: click.Context) -> list[str]:
        """Returns a list of template names from the template filenames."""
        rv = []
        if self.TEMPLATES_DIR.is_dir():
            for cmd in self.TEMPLATES_DIR.iterdir():
                if cmd.stem.lower() not in ("__init__", "group") and cmd.suffix.lower() == ".py":
                    rv.append(cmd.stem)
        if not rv:
            raise click.ClickException(f"No Templates were found in {Directories.templates}")
        rv.sort()
        return rv

    def get_command(self, ctx: click.Context, name: str) -> Optional[click.Command]:
        """Load the template code and return the main click command function."""
        scope: dict[str, Any] = {}
        fn = self.TEMPLATES_DIR / f"{name}.py"
        if not fn.exists():
            raise click.ClickException(f"The Template ({name}) was not found in {Directories.templates}.")
        with fn.open(encoding="utf8") as f:
            code = compile(f.read(), fn, "exec")
            eval(code, scope, scope)
        return scope[name].cli



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nfog', 'nfog.artwork', 'nfog.templates', 'nfog.tracks']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'cinemagoer>=2022.1.26,<2023.0.0',
 'click-default-group>=1.2.2,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'jsonpickle>=2.1.0,<3.0.0',
 'langcodes[data]>=3.3.0,<4.0.0',
 'pyd2v>=1.3.0,<2.0.0',
 'pymediainfo>=5.1.0,<6.0.0',
 'requests>=2.27.1,<3.0.0',
 'tmdbsimple>=2.9.1,<3.0.0']

entry_points = \
{'console_scripts': ['nfo = nfog.nfog:cli']}

setup_kwargs = {
    'name': 'nfog',
    'version': '1.1.0',
    'description': 'Scriptable Database-Driven NFO Generator for Movies and TV.',
    'long_description': "# nfog\n\n[![License](https://img.shields.io/github/license/rlaphoenix/nfog)](https://github.com/rlaphoenix/nfog/blob/master/LICENSE)\n[![Python Support](https://img.shields.io/pypi/pyversions/nfog)](https://pypi.python.org/pypi/nfog)\n[![Release](https://img.shields.io/pypi/v/nfog)](https://pypi.python.org/pypi/nfog)\n[![GitHub issues](https://img.shields.io/github/issues/rlaphoenix/nfog)](https://github.com/rlaphoenix/nfog/issues)\n\nScriptable Database-Driven NFO Generator for Movies and TV.\n\n## Installation\n\n    pip install --user nfog\n\n## Building\n\n### Dependencies\n\n- [Python](https://python.org/downloads) (v3.7 or newer)\n- [Poetry](https://python-poetry.org/docs) (latest recommended)\n\n### Installation\n\n1. `git clone https://github.com/rlaphoenix/nfog`\n2. `cd nfog`\n3. `poetry config virtualenvs.in-project true` (optional, but recommended)\n4. `poetry install`\n5. `nfo -h`\n\n## Creating Templates\n\nWe use Template's to define the structure and logic that creates your NFO file. Your Template file may\ncreate NFOs of any kind of encoding or style, including ASCII, ANSI, and such. You don't have to conform\nto any specifications of any kind, but are encouraged to if possible.\n\nTo create a Template file, you simply need to inherit the `Template` class in `nfog.template`, fill out\nthe various abstract methods/properties, and create an `nfo` property that returns a final string.\n\nTake a look at the [Example Templates](/examples/templates) for pre-made examples for various NFO\nusage scenarios. You may modify these Templates in any way you like.\n\nNote: While you have complete freedom with what Python code you run from within the template, this also\nmeans you should not immediately trust template file as they are after all still Python files.\n\n## Creating Artwork\n\nJust like Templates, we use Artwork files to define the look and style of the surrounding NFO.\nYou may also do introspection of the NFO output to merge style within the contents of the NFO as well.\n\nTo create an Artwork file, inherit the `Artwork` class in `nfog.artwork`, fill out any abstract methods\nand properties, and create the `with_template` function that returns the final string containing both\nthe NFO output (from `template` argument) and the Artwork.\n\nTake a look at the [Example Artwork](/examples/artwork) to see how these are used. However, you cannot\nre-use these, or make derivative works. Please see the [Artwork License](/examples/artwork/LICENSE)\nfor more information.\n\n## Using Templates and Artwork\n\nTo use Templates and Artwork, calling `nfo` (or `nfo generate`) will ask you for various information, but\none of them is a Template to use. The Templates it makes available to use are loaded from the user templates\ndirectory which can be found by typing `nfo version`.\n\nTo use an Artwork, specify the name of the Artwork file (case-sensitive) to `-a/--artwork`.\nUsing an Artwork is completely optional.\n\nFor more information on using `nfog`, see the usage help by calling `nfo --help`.\n\n## License\n\n[Apache License, Version 2.0](LICENSE)\n",
    'author': 'PHOENiX',
    'author_email': 'rlaphoenix@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rlaphoenix/nfog',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from abc import abstractmethod

from nfog.templates import Template


class Artwork:
    @staticmethod
    @abstractmethod
    def with_template(template: Template) -> str:
        """Return the Artwork with the Template information."""



from pathlib import Path

import tmdbsimple
import toml
from appdirs import user_data_dir


class Directories:
    user_data = Path(user_data_dir("nfog", False))
    templates = user_data / "templates"
    artwork = user_data / "artwork"


class Files:
    config = Directories.user_data / "config.toml"
    template = lambda name: Directories.templates / f"{name}.py"  # noqa: E731
    artwork = lambda name: Directories.artwork / f"{name}.py"  # noqa: E731


if Files.config.exists():
    config = toml.load(Files.config)
else:
    config = {}

tmdbsimple.API_KEY = config.get("api-keys").get("tmdb")

__ALL__ = (config, Directories, Files)



from __future__ import annotations

from pathlib import Path

import pymediainfo

from nfog.tracks.BaseTrack import BaseTrack


class Audio(BaseTrack):
    AUDIO_CHANNEL_LAYOUT_WEIGHT = {
        "LFE": 0.1
    }

    def __init__(self, track: pymediainfo.Track, path: Path):
        super().__init__(track, path)

    @property
    def codec(self) -> str:
        """
        Get track codec in common P2P simplified form.
        E.g., 'DD+' instead of 'E-AC-3'.
        """
        return {
            "E-AC-3": "DD+",
            "AC-3": "DD"
        }.get(self._x.format, self._x.format)

    @property
    def channels(self) -> float:
        """Get track channels as channel layout representation."""
        if self._x.channel_layout:
            return float(sum(
                self.AUDIO_CHANNEL_LAYOUT_WEIGHT.get(x, 1)
                for x in self._x.channel_layout.split(" ")
            ))
        return float(self._x.channel_s)


__ALL__ = (Audio,)
